/*****************************************************
Chip type               : ATmega8
AVR Core Clock frequency: 8,000000 MHz
*****************************************************/

#include <mega8.h>
#include <STDLIB.h>

#define A PORTD.7
#define B PORTD.6
#define C PORTB.6
#define D PORTD.4
#define E PORTD.3
#define F PORTB.0
#define G PORTB.1
#define DP PORTB.7
#define A1 PORTC.4
#define A2 PORTD.0
#define A3 PORTD.1
#define B1 PORTC.5
#define B2 PORTD.5
#define B3 PORTD.2

#define button0    PINB.3 // mode
#define button1    PINB.4 //key_up 
#define button2    PINB.5 // key_dn 

#define ADC_itn 16
#define shunt_res 25 //mOhms
#define amplification 10 //4LM358 

enum btn_states 
{b_prs, b_rls, b_hold} btn0, btn1, btn2;
enum modes 
{LU_adj, HU_adj, I_adj, normal_operation}mode = normal_operation;

eeprom signed int LU_corr, HU_corr, I_corr;
signed int LU_corr_tmp, HU_corr_tmp, I_corr_tmp;
signed int ADC0_tmp [ADC_itn], ADC1_tmp [ADC_itn], ADC2_tmp [ADC_itn];
signed int ADC0_fr, ADC1_fr, ADC2_fr; 
char display_counter, display_buff [6], d_point_position; 

void ports_init ()
{
 DDRB = 0b11000111; PORTB = 0b00111000;  
 DDRC = 0b00111000; PORTC = 0b01000000;
 DDRD = 0b11111111; PORTD = 0b00000000;
}

void ADC_init ()
{
 MCUSR |= 1<<SE|1<<SM0; //adc noise reduction 
 ADCSR |= 1<<ADEN|1<<ADIE|1<<ADPS2|1<<ADPS1|1<<ADPS0; // div by 128
 ADMUX |= 1<<REFS1|1<<REFS0; //2,56 bg refs 
 ACSR &= ~(1<<ACD); //disable comparator
}

void convert_data ()
{
 signed int volts, amps;      
 char i=0;

if (ADC0_fr < 1020) 
{
volts =  (((long)ADC0_fr * 10000 - (((long)ADC2_fr*25*1024)/100))>>10)/10;
if (volts > 0) {;} else {volts = 0;} 
d_point_position = 2;
}
else 
{
volts = (((long)ADC1_fr * 25000 - (((long)ADC2_fr*25*1024)/100))>>10)/100;  
if (volts > 0) {;} else {volts = 0;} 
d_point_position = 1;
}

amps = ((long)ADC2_fr*256*100)/(shunt_res*amplification*1024);

for (i=0; i<3; i++) {display_buff [i] = volts%10; volts /= 10;}
for (i=3; i<6; i++) {display_buff [i] = amps%10; amps /= 10;}
}

void clear_display () 
{ A=1; B=1; C=1; D=1; E=1; F=1; G=1; DP=1; A1 = 0; A2 = 0; A3 = 0, B1 = 0; B2 = 0; B3 = 0; }
 
void display_data ()
{

 if (display_counter < 5) {display_counter++;} else {display_counter = 0;}
 
 switch (display_buff[display_counter])
 {
  case 0: A=0; B=0; C=0; D=0; E=0; F=0; G=1; break;
  case 1: A=1; B=0; C=0; D=1; E=1; F=1; G=1; break;
  case 2: A=0; B=0; C=1; D=0; E=0; F=1; G=0; break;
  case 3: A=0; B=0; C=0; D=0; E=1; F=1; G=0; break;
  case 4: A=1; B=0; C=0; D=1; E=1; F=0; G=0; break;
  case 5: A=0; B=1; C=0; D=0; E=1; F=0; G=0; break;
  case 6: A=0; B=1; C=0; D=0; E=0; F=0; G=0; break;
  case 7: A=0; B=0; C=0; D=1; E=1; F=1; G=1; break;
  case 8: A=0; B=0; C=0; D=0; E=0; F=0; G=0; break;
  case 9: A=0; B=0; C=0; D=0; E=1; F=0; G=0; break;
  
  /*case 10: A=0; B=0; C=0; D=0; E=0; F=0; G=1; DP = 0; break;
  case 11: A=1; B=0; C=0; D=1; E=1; F=1; G=1; DP = 0; break;
  case 12: A=0; B=0; C=1; D=0; E=0; F=1; G=0; DP = 0; break;
  case 13: A=0; B=0; C=0; D=0; E=1; F=1; G=0; DP = 0; break;
  case 14: A=1; B=0; C=0; D=1; E=1; F=0; G=0; DP = 0; break;
  case 15: A=0; B=1; C=0; D=0; E=1; F=0; G=0; DP = 0; break;
  case 16: A=0; B=1; C=0; D=0; E=0; F=0; G=0; DP = 0; break;
  case 17: A=0; B=0; C=0; D=1; E=1; F=1; G=1; DP = 0; break;
  case 18: A=0; B=0; C=0; D=0; E=0; F=0; G=0; DP = 0; break;
  case 19: A=0; B=0; C=0; D=0; E=1; F=0; G=0; DP = 0; break;*/
  
  case 20: A=1; B=1; C=1; D=0; E=0; F=0; G=1; break;     //L
  case 21: A=1; B=0; C=0; D=1; E=0; F=0; G=0; break;     //H
  case 22: A=1; B=1; C=1; D=1; E=1; F=1; G=1; break;     //
  default : A=1; B=1; C=1; D=1; E=1; F=1; G=0; break;    //-  
   }
 
 switch (display_counter)
 {
  case 0: A1 = 1; if(d_point_position == 0) {DP = 0;}break;
  case 1: A2 = 1; if(d_point_position == 1) {DP = 0;} break;
  case 2: A3 = 1; if(d_point_position == 2) {DP = 0;} break; 
  case 3: B1 = 1; break;
  case 4: B2 = 1; DP = 0; break;
  case 5: if (!display_buff[5]) {B3 = 0;} else {B3 = 1;} break;
 }
  
}

void skan_keys ();

void modes_selector ();

interrupt [ADC_INT] void adc_isr()
{
 static char ADC_cnl, ADC_select;
 static char i = 0;
 
 if (!ADC_cnl)
 {
  for (i=0; i<(ADC_itn-1); i++ ) {ADC0_tmp [i] = ADC0_tmp [i+1];}
  ADC0_tmp [(ADC_itn-1)] = ADCW;
  ADC0_fr = 0;
  for (i=0; i<(ADC_itn); i++ ) { ADC0_fr += ADC0_tmp [i];}
  ADC0_fr /= ADC_itn;
  ADC0_fr += LU_corr_tmp;
  if (ADC0_fr > 0) {;} else {ADC0_fr = 0;}
 }
 else if (ADC_cnl == 1)
 {
  for (i=0; i<(ADC_itn-1); i++ ) {ADC1_tmp [i] = ADC1_tmp [i+1];}
  ADC1_tmp [(ADC_itn-1)] = ADCW;
  ADC1_fr = 0;
  for (i=0; i<(ADC_itn); i++ ) { ADC1_fr += ADC1_tmp [i];}
  ADC1_fr /= ADC_itn;
  ADC1_fr += HU_corr_tmp;
   if (ADC1_fr > 0) {;} else {ADC1_fr = 0;}
 }
 else
 {
  for (i=0; i<(ADC_itn-1); i++ ) {ADC2_tmp [i] = ADC2_tmp [i+1];}
  ADC2_tmp [(ADC_itn-1)] = ADCW;
  ADC2_fr = 0;
  for (i=0; i<(ADC_itn); i++ ) { ADC2_fr += ADC2_tmp [i];}
  ADC2_fr /= ADC_itn;
  ADC2_fr += I_corr_tmp;
  if (ADC2_fr > 0) {;} else {ADC2_fr = 0;}
 } 
   
   switch (ADC_select)
    {
    case 0: ADMUX &= 0b11111000;                     ADC_select = 1; ADC_cnl = 0; break;
    case 1: ADMUX &= 0b11111000;  ADMUX |= 1; ADC_select = 2; ADC_cnl = 1; break;
    case 2: ADMUX &= 0b11111000;  ADMUX |= 2; ADC_select = 0; ADC_cnl = 2; break;
    default: break;
    }
  
    skan_keys ();
    modes_selector ();
    
    #asm("wdr")
    ADCSR |= 1<<ADSC;  
    //#asm("sleep")
}


void main(void)
{
ports_init (); 
ADC_init ();

LU_corr_tmp = LU_corr;
HU_corr_tmp = HU_corr;
I_corr_tmp = I_corr;

#pragma optsize-
WDTCR=0x1E;
WDTCR=0x0E;
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif

#asm("sei")

 ADCSR |= 1<<ADSC;

while (1){#asm("sleep")}
}

void skan_keys ()
{
static unsigned int count0, count1, count2, count3;

if (button0){ btn0 = b_rls;}
if (button1){ btn1 = b_rls;}
if (button2){ btn2 = b_rls;}

if (!button0) {count0++; } 
else {count0 = 0;}
if (!button1) {count1++; } 
else {count1 = 0;}
if (!button2) {count2++; } 
else {count2 = 0;}

if ((count0 > 155)&&(btn0 != b_hold)) {btn0 = b_prs; count0 = 0; count3++;}
if ((count1 > 155)&&(btn1 != b_hold)) {btn1 = b_prs; count1 = 0; count3++;} 
if ((count2 > 155)&&(btn2 != b_hold)) {btn2 = b_prs; count2 = 0; count3++;}

if ((count3 > 25)&&(btn0 == b_prs)) {btn0 = b_hold; count3 = 0;}
if ((count3 > 25)&&(btn1 == b_prs)) {btn1 = b_hold; count3 = 0;}
if ((count3 > 25)&&(btn2 == b_prs)) {btn2 = b_hold; count3 = 0;}
}

void modes_selector ()
{
char n, tmp;
static int counter3;
static bit trigger0, trigger1, trigger2,  
ee_trigger_LU, ee_trigger_HU, ee_trigger_I;

if (button0) {trigger0 = 0;}                                  
if (button1) {trigger1 = 0;}                                    
if (button2) {trigger2 = 0;}

if ((btn0 == b_hold)&&(!trigger0)) {trigger0 = 1; if (mode < normal_operation) {mode++; } else {mode = LU_adj;}}

switch (mode)
{
 case LU_adj: 
 {
    
  if ((btn1 == b_prs)&&(!trigger1)) {trigger1 = 1; if (LU_corr_tmp < 99) {LU_corr_tmp++;} else {LU_corr_tmp = 0;}}
  if ((btn2 == b_prs)&&(!trigger2)) {trigger2 = 1; if (LU_corr_tmp > -99) {LU_corr_tmp--; } else {LU_corr_tmp = 0;}}
  
  clear_display ();
  if (LU_corr_tmp>0) {display_buff[2] = 21;} 
  else if (LU_corr_tmp<0) {display_buff[2] = 20;}
  else  {display_buff[2] = 23;}
  tmp = abs (LU_corr_tmp);
  
  for (n=0; n<2; n++) {display_buff [n] = tmp % 10; tmp /= 10;}
  d_point_position = 1;
  display_data ();
  ee_trigger_LU =0;
 }   
 break;
 
 case HU_adj: 
 {
  if ((btn1 == b_prs)&&(!trigger1)) {trigger1 = 1; if (HU_corr_tmp < 99) {HU_corr_tmp++;} else {HU_corr_tmp = 0;}}
  if ((btn2 == b_prs)&&(!trigger2)) {trigger2 = 1; if (HU_corr_tmp > -99) {HU_corr_tmp--; } else {HU_corr_tmp = 0;}}
  
  clear_display ();
  if (HU_corr_tmp>0) {display_buff[2] = 21;} 
  else if (HU_corr_tmp<0) {display_buff[2] = 20;}
  else  {display_buff[2] = 23;}
  tmp = abs (HU_corr_tmp);
  for (n=0; n<2; n++) {display_buff [n] = tmp % 10; tmp /= 10;}
  d_point_position = 2;
  display_data ();
  ee_trigger_HU =0; 
 }   
 break;
 case I_adj: 
 {
  if ((btn1 == b_prs)&&(!trigger1)) {trigger1 = 1; if (I_corr_tmp < 99) {I_corr_tmp++;} else {I_corr_tmp = 0;}}
  if ((btn2 == b_prs)&&(!trigger2)) {trigger2 = 1; if (I_corr_tmp > -99) {I_corr_tmp--; } else {I_corr_tmp = 0;}}
  
  clear_display ();
  if (I_corr_tmp>0) {display_buff[5] = 21;} 
  else if (I_corr_tmp<0) {display_buff[5] = 20;}
  else  {display_buff[5] = 23;}
  tmp = abs (I_corr_tmp);
  for (n=3; n<5; n++) {display_buff [n] = tmp % 10; tmp /= 10;}
  display_data ();
  ee_trigger_I =0;  
 }   
 break;
 case normal_operation:
 {
  if (!ee_trigger_LU) {ee_trigger_LU = 1; if (LU_corr != LU_corr_tmp) {LU_corr = LU_corr_tmp;}}
  if (!ee_trigger_HU) {ee_trigger_HU = 1; if (HU_corr != HU_corr_tmp) {HU_corr = HU_corr_tmp;}}
  if (!ee_trigger_I)    {ee_trigger_I = 1; if (I_corr != I_corr_tmp) {I_corr = I_corr_tmp;}}
  
  clear_display ();  
  if (counter3) {counter3--;}  else { counter3 = 1000; convert_data ();}
   
  display_data (); 
 }   
 break;
 
 default: break;
}
}
